@@ -49,10 +49,17 @@ Node *newNode()
 
 void deleteNode(Node * n)
 {
+    char *tmp = NULL;
+
     if (n != NULL) {
 	if (n->copy == 0) {
 	    if ((n->remove == 1) || (n->remove == 0 && n->play == 0)) {
 		if (strlen(n->fname) != 0) {
+		    if (strncasecmp(n->fname, "sdp://", 6) == 0) {
+			tmp = strdup(n->fname);
+			strcpy(n->fname, tmp + 6);
+			free(tmp);
+		    }
 		    remove(n->fname);
 		    if (DEBUG)
 			printf("fname: %s deleted\n", n->fname);
@@ -252,7 +259,7 @@ void addToList(nsPluginInstance * instance, char *item, Node * parent,
 	}
 	addToEnd(parent, node);
 	parent->play = 0;
-	if (isMms(item)) {
+	if (isMms(item) || strstr(node->url, parent->url) != NULL) {
 	    node->mmsstream = 1;
 	} else {
 	    filename = getURLFilename(localitem);
@@ -265,7 +272,7 @@ void addToList(nsPluginInstance * instance, char *item, Node * parent,
 	    }
 	    if (filename != NULL)
 		NPN_MemFree(filename);
-	    NPN_GetURL(instance->mInstance, node->url, NULL);
+	    NPN_GetURLNotify(instance->mInstance, node->url, NULL, NULL);
 	}
     } else {
 	if (n->playlist == 1)
@@ -280,14 +287,12 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
     char buffer[16 * 1024];
     char buffer_lower[16 * 1024];
     char url[1024];
-    int size, found;
+    unsigned long int size, found, i, remainder;
     char *p;
     char *nextrmda = NULL;
     char *rdrf;
     char *rmdr;
     char *sub, *refurl;
-    int i;
-    int remainder;
     char *seqtag, *rate, *videotag, *videosrc;
     unsigned int code;
     int speed;
@@ -306,7 +311,7 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 	    buffer[size] = '\0';
 
 	    if (DEBUG > 1)
-		printf("READ: %i\n%s\n", size, buffer);
+		printf("READ: %li\n%s\n", size, buffer);
 
 	    // quicktime files have blocks of 4 characters..
 	    // rdrf,XXXX,url ,XXXX,url ending with 00
@@ -329,7 +334,8 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 		speed = -1;
 		code = 0;
 
-		while ((p != NULL) && ((long) p < ((long) buffer + size))) {
+		while ((p != NULL)
+		       && ((long) p < ((long) buffer + (long) size))) {
 		    rdrf =
 			(char *) memmem(p,
 					size - ((long) nextrmda -
@@ -522,7 +528,7 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 		if (DEBUG)
 		    printf("found QuickTime Media Link file\n");
 		while (p != NULL) {
-		    i = size - ((long) buffer_lower - (long) p);
+		    i = size - ((long) p - (long) buffer_lower);
 		    if (i < 1)
 			break;
 		    p = (char *) memmem(p, i, "src", 3);
@@ -568,12 +574,33 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 		if (DEBUG)
 		    printf("found ASX file\n");
 		while (p != NULL) {
-		    i = size - ((long) buffer_lower - (long) p);
+		    i = size - ((long) p - (long) buffer_lower);
+
 		    if (i < 1)
 			break;
 		    p = (char *) memmem(p, i, "href", 4);
 		    if (p == NULL)
 			break;
+		    if ((p - 9) > buffer_lower) {
+			if (memmem(p - 5, i, "logo", 4) != NULL) {
+			    if (DEBUG > 1)
+				printf("found logo in ASX\n");
+			    p = p + 4;
+			    continue;
+			}
+			if (memmem(p - 7, i, "banner", 6) != NULL) {
+			    if (DEBUG > 1)
+				printf("found banner in ASX\n");
+			    p = p + 4;
+			    continue;
+			}
+			if (memmem(p - 9, i, "moreinfo", 8) != NULL) {
+			    if (DEBUG > 1)
+				printf("found moreinfo in ASX\n");
+			    p = p + 4;
+			    continue;
+			}
+		    }
 		    p = p + 4;
 		    p = strchr(p, '\"');
 		    if (p != NULL) {
@@ -607,7 +634,7 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 		lowercase(buffer_lower);
 		p = buffer_lower;
 		while (p != NULL) {
-		    i = size - ((long) buffer_lower - (long) p);
+		    i = size - ((long) p - (long) buffer_lower);
 		    if (i < 1)
 			break;
 		    p = (char *) memmem(p, i, "ref", 3);
@@ -652,6 +679,12 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 	    if (memmem(buffer_lower, size, "<html", 5) != NULL) {
 		found = 1;
 	    }
+	    // we have an NSV file
+	    if (memmem(buffer_lower, size, "icy 200 ok", 10) != NULL) {
+		parent->mmsstream = 1;
+		parent->cancelled = 1;
+		found = 1;
+	    }
 	    // simple playlist usually realmedia file
 	    if (strncasecmp(buffer, "rtsp://", 7) == 0 && found == 0) {
 		p = buffer;
