@@ -177,7 +177,9 @@ void play_callback(GtkWidget * widget, GdkEventExpose * event,
 void pause_callback(GtkWidget * widget, GdkEventExpose * event,
 		   nsPluginInstance * instance);
 void stop_callback(GtkWidget * widget, GdkEventExpose * event,
-		   nsPluginInstance * instance);		   		   
+		   nsPluginInstance * instance);
+gboolean keyboard_callback(GtkWidget *widget, GdkEventKey *event,
+	               	   nsPluginInstance * instance);
 gboolean gtkgui_draw(void *data);
 gboolean gtkgui_message(void *data);
 gboolean gtkgui_progress(void *data);
@@ -186,6 +188,7 @@ gboolean gtkgui_stop(void *data);
 gboolean gtkgui_save_enable(void *data);
 gboolean gtkgui_drawMediaProgress(void *data);
 gboolean gtkgui_updatebuttons(void *data);
+gboolean gtkgui_updatefullscreen(void *data);
 gint popup_handler(GtkWidget *widget, GdkEvent *event);
 void menuitem_play_callback(GtkMenuItem *menuitem, nsPluginInstance* instance);
 void menuitem_pause_callback(GtkMenuItem *menuitem, nsPluginInstance* instance);
@@ -194,6 +197,8 @@ void menuitem_showcontrols_callback(GtkCheckMenuItem *menuitem, nsPluginInstance
 void menuitem_fullscreen_callback(GtkCheckMenuItem *menuitem, nsPluginInstance* instance);
 void menuitem_save_callback(GtkMenuItem * menuitem, nsPluginInstance * instance);
 void menuitem_copy_callback(GtkMenuItem * menuitem, nsPluginInstance * instance);
+void menuitem_config_callback(GtkMenuItem * menuitem, nsPluginInstance * instance);
+gboolean window_visible(GtkWidget *widget,GdkEvent *event,nsPluginInstance * instance);
 #endif
 
 //plugin-threads.cpp
