@@ -78,7 +78,12 @@ char *getURLHostname(char *url)
 	    break;
 	}
     }
-    hostname[i] = '\0';
+    if (i == 0) {
+	NPN_MemFree(hostname);
+	hostname = NULL;
+    } else {
+	hostname[i] = '\0';
+    }
     if (DEBUG > 1)
 	printf("exiting getURLHostname with %s\n", hostname);
 
@@ -107,7 +112,7 @@ char *getURLFilename(char *url)
 	strcpy(filename, url);
 	return filename;
     }
-    strcpy(filename, tmp);
+    strcpy(filename, tmp + 1);	// take everything upto, but not including the / starting from the right.
     return filename;
 }
 
@@ -127,7 +132,7 @@ char *getURLBase(char *url)
 	return NULL;
     base = (char *) NPN_MemAlloc(sizeof(char) * (len + 1));
     strcpy(base, url);
-    if (0)
+    if (DEBUG > 1)
 	printf("in getURLBase base: %s\n", base);
 
     for (i = len - 1; i >= 0; i--) {
@@ -137,7 +142,7 @@ char *getURLBase(char *url)
 	    break;
 	}
     }
-    if (strlen(base) == 0) {
+    if ((strlen(base) == 0) || (i <= 0)) {
 	NPN_MemFree(base);
 	base = NULL;
     }
@@ -153,18 +158,20 @@ char *getURLBase(char *url)
 
 int isMms(char *url)
 {
-    if (DEBUG > 1)
-	printf("in isMms\n");
-
     if (url == NULL)
 	return 0;
     if ((strncasecmp(url, "mms://", 6) == 0)
 	|| (strncasecmp(url, "mmst://", 7) == 0)
 	|| (strncasecmp(url, "mmsu://", 7) == 0)
 	|| (strncasecmp(url, "dvd://", 6) == 0)
+	|| (strncasecmp(url, "smb://", 6) == 0)
 	|| (strncasecmp(url, "rtsp://", 7) == 0)) {
+	if (DEBUG > 1)
+	    printf("isMms = true\n");
 	return 1;
     } else {
+	if (DEBUG > 1)
+	    printf("isMms = false\nurl = %s\n", url);
 	return 0;
     }
 }
@@ -192,8 +199,12 @@ int URLcmp(const char *url1, const char *url2)
     char *tmp2;
     char *q1;			// question mark in tmp
     char *q2;			// question mark in tmp2
-    char *hostname1;		// hostname1
-    char *hostname2;		// hostname2
+    int tmp1hasq = 0;		// tmp1 has a ?
+    int tmp2hasq = 0;		// tmp2 has a ?
+    char *hostname1 = NULL;	// hostname1
+    char *hostname2 = NULL;	// hostname2
+    char *protocol1 = NULL;
+    char *protocol2 = NULL;
     int ret;
 
     if (DEBUG > 1)
@@ -234,6 +245,21 @@ int URLcmp(const char *url1, const char *url2)
 	ret = -1;
     }
 
+    if (strncasecmp(buffer1, "file://", 7) == 0) {
+	if (strcmp(buffer1 + 7, buffer2) == 0) {
+	    free(buffer1);
+	    free(buffer2);
+	    ret = 0;
+	}
+    }
+
+    if (strncasecmp(buffer2, "file://", 7) == 0) {
+	if (strcmp(buffer1, buffer2 + 7) == 0) {
+	    free(buffer1);
+	    free(buffer2);
+	    ret = 0;
+	}
+    }
 
     if (ret == -1) {
 	hostname1 = getURLHostname(buffer1);
@@ -242,12 +268,30 @@ int URLcmp(const char *url1, const char *url2)
 	if (hostname1 != NULL && hostname2 != NULL
 	    && strstr(hostname2, hostname1) == NULL) {
 	    // hostname1 is a not substring of hostname2
+	    // this is done for the case where the playlist has two urls that basically point to the same site
+	    // one of the sites is an internal URL and the other is an external. We want to be able to play either
+	    // site. So as long as the hostnames are completly different we should be ok. 
+	    if (DEBUG > 1)
+		printf("URLcmp: hostnames do not match\n");
 	    ret = -1;
 	} else {
 	    // url1 is a substring of url2, so continue comparing
-
+	    if (DEBUG > 1)
+		printf("hostname1 = %s\nhostname2 = %s\n", hostname1,
+		       hostname2);
 	    // compare the paths, some sites change the hostname mid stream (like cartoonnetwork.com -> www.cartoonnetwork.com);
 	    tmp1 = strstr(buffer1, "://");
+	    if (tmp1 != NULL) {
+		protocol1 =
+		    (char *) malloc((long) tmp1 - (long) buffer1 + 1);
+		strncpy(protocol1, buffer1,
+			(long) tmp1 - (long) buffer1 + 1);
+		protocol1[(long) tmp1 - (long) buffer1] = '\0';
+	    }
+
+	    if (DEBUG > 1)
+		printf("protocol1: %s\n", protocol1);
+
 	    if (tmp1 != NULL) {
 		tmp1 = tmp1 + 3;
 		while (tmp1[0] != '/') {
@@ -257,6 +301,17 @@ int URLcmp(const char *url1, const char *url2)
 		}
 	    }
 	    tmp2 = strstr(buffer2, "://");
+	    if (tmp2 != NULL) {
+		protocol2 =
+		    (char *) malloc((long) tmp2 - (long) buffer2 + 1);
+		strncpy(protocol2, buffer2,
+			(long) tmp2 - (long) buffer2 + 1);
+		protocol2[(long) tmp2 - (long) buffer2] = '\0';
+	    }
+
+	    if (DEBUG > 1)
+		printf("protocol2: %s\n", protocol2);
+
 	    if (tmp2 != NULL) {
 		tmp2 = tmp2 + 3;
 		while (tmp2[0] != '/') {
@@ -267,18 +322,41 @@ int URLcmp(const char *url1, const char *url2)
 	    }
 	    if (tmp1 != NULL && tmp2 != NULL) {
 		if (strcmp(tmp1, tmp2) == 0) {
-		    ret = 0;
+		    // if either protocol is file:// then they are the same at this point usually one will be NULL and the other file://
+		    if ((strncmp(protocol1, "file://", 7) == 0)
+			|| (strncmp(protocol2, "file://", 7) == 0)) {
+			ret = 0;
+		    } else {
+			// NetFlix puts out the same preview on multiple protocols, we have to add them all to the playlist
+			if (strcmp(protocol1, protocol2) == 0) {
+			    ret = 0;
+			} else {
+			    ret = -1;
+			}
+		    }
 		} else {
 		    ret = -1;
 		    q1 = strchr(tmp1, '?');
 		    q2 = strchr(tmp2, '?');
 		    if (q1 != NULL || q2 != NULL) {
-			if (q1 != NULL)
+			if (q1 != NULL) {
 			    q1[0] = '\0';
-			if (q2 != NULL)
+			    tmp1hasq = 1;
+			}
+			if (q2 != NULL) {
 			    q2[0] = '\0';
+			    tmp2hasq = 1;
+			}
 			if (strcmp(tmp1, tmp2) == 0) {
-			    ret = 0;
+			    if (tmp1hasq != tmp2hasq) {
+				ret = -1;
+			    } else {
+				if (strcmp(q1+1,q2+1) == 0) {
+				    ret = 0;
+				} else { 
+				    ret = -1;
+				}
+			    }
 			} else {
 			    ret = -1;
 			}
@@ -286,11 +364,21 @@ int URLcmp(const char *url1, const char *url2)
 		}
 	    }
 	}
-	NPN_MemFree(hostname1);
-	NPN_MemFree(hostname2);
 	free(buffer1);
 	free(buffer2);
+
     }
+    if (hostname1 != NULL)
+	NPN_MemFree(hostname1);
+    if (hostname2 != NULL)
+	NPN_MemFree(hostname2);
+    if (protocol1 != NULL)
+	free(protocol1);
+    if (protocol2 != NULL)
+	free(protocol2);
+
+    if (DEBUG > 1)
+	printf("exiting URLcmp\n");
 
     return ret;
 
@@ -338,7 +426,7 @@ int sendCommand(nsPluginInstance * instance, char *command)
     char buffer[1024];
 
     if (DEBUG > 1)
-	printf("in sendcommand\n");
+	printf("in sendcommand - command %s\n", command);
 
     buffer[1023] = '\0';
     retval = 0;
@@ -346,6 +434,13 @@ int sendCommand(nsPluginInstance * instance, char *command)
     if (command == NULL || instance == NULL || instance->cancelled == 1)
 	return 0;
 
+    if (instance->threadsignaled == 0)
+	return 0;
+
+    if (instance->control == -1)
+	return 0;
+
+
     if (instance->js_state != JS_STATE_TRANSITIONING) {
 
 	snprintf(buffer, 1023, "%s\n", command);
@@ -397,6 +492,8 @@ void killmplayer(nsPluginInstance * instance)
 
     count = 0;
     while (instance->player != NULL && count < 10) {
+	if (DEBUG)
+	    printf("waiting for player to go NULL\n");
 	usleep(100);
 	count++;
     }
@@ -405,17 +502,22 @@ void killmplayer(nsPluginInstance * instance)
     if (instance->player == NULL) {
 	instance->pid = 0;
     } else {
-/*	
-	fclose(instance->player);
+	if (DEBUG > 1)
+	    printf("closing player\n");
+//      fclose(instance->player);
 	instance->player = NULL;
 
+	if (DEBUG > 1)
+	    printf("closing control pipe\n");
 	if (instance->control > 0) {
 	    close(instance->control);
 	    instance->control = -1;
 	}
-*/
     }
 
+    if (DEBUG > 1)
+	printf("player should be closed\n");
+
     if (instance->pid != 0) {
 	count = 0;
 	status = 1;
@@ -445,7 +547,7 @@ void killmplayer(nsPluginInstance * instance)
 	}
 	//wait(&status);
     }
-#ifdef DPMSEnabled
+#ifdef DPMSExtension
     if (instance->DPMSEnabled)
 	DPMSReenable(instance);
 #endif
@@ -478,22 +580,35 @@ void fullyQualifyURL(nsPluginInstance * instance, char *item,
 	if ((strncasecmp(item, "http", 4) != 0)
 	    && (strncasecmp(item, "file", 4) != 0)) {
 
+	    if (DEBUG > 1)
+		printf("not http and not file\n");
+
 	    if (item[0] != '/') {
 		strlcpy(tmpdir, item, 1024);	// reuse the buffer
 		if (instance->baseurl != NULL) {
 		    strlcpy(localitem, instance->baseurl, 1024);
+		} else {
+		    strlcpy(localitem, "", 1024);
 		}
 		strlcat(localitem, tmpdir, 1024);
 	    } else {
 		if (instance->hostname != NULL) {
-		    snprintf(tmpdir, 1024, "http://%s%s",
-			     instance->hostname, item);
-		    strlcpy(localitem, tmpdir, 1024);
+		    if (fexists(item) == 0) {
+			snprintf(tmpdir, 1024, "http://%s%s",
+				 instance->hostname, item);
+			strlcpy(localitem, tmpdir, 1024);
+		    } else {
+			// first char == / and hostname is null, sounds like a filename
+			strlcpy(localitem, item, 1024);
+		    }
+		} else {
+		    // first char == / and hostname is null, sounds like a filename
+		    strlcpy(localitem, item, 1024);
 		}
 	    }
 
-
 	} else {
+
 	    // if :80 is in the URL, cut it out
 	    strlcpy(localitem, item, 1024);
 	    tmp = strstr(localitem, ":8080");
@@ -520,14 +635,26 @@ void fullyQualifyURL(nsPluginInstance * instance, char *item,
 			}
 		    }
 		}
+		// check and see if the file exists, if it does not then prepend smb:// to the filename
+		if (fexists(localitem) == 0) {
+		    strlcpy(tmpdir, "smb://", 1024);
+		    strlcat(tmpdir, localitem, 1024);
+		    strlcpy(localitem, tmpdir, 1024);
+		}
+
 	    }
+
 	}
+
     } else {
+
 	strlcpy(localitem, item, 1024);
+
     }
 
     if (DEBUG >= 2)
 	printf("fqu result: %s\n", localitem);
+
 }
 
 int toolkitOk(NPP instance, int *mozilla_toolkit, int *plugin_toolkit)
