@@ -6,6 +6,7 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <sys/wait.h>
+#include <sys/param.h>
 
 extern int DEBUG;
 
@@ -183,15 +184,22 @@ FILE *mypopen(char **argv, pid_t * pid, int *control)
 	close(filedesr[0]);
 	setsid();
 	setpgid(0, 0);
-
+	sigemptyset(&newmask);
+	sigaddset(&newmask, SIGTERM);
+	sigaddset(&newmask, SIGKILL);
+	pthread_sigmask(SIG_UNBLOCK, &newmask, 0L);
+	usleep(500);
 	if (execvp(argv[0], argv) < 0)
 	    perror("execv");
 	_exit(0);
+
     } else {
 	// setup signal handler for child
 	signal(SIGCHLD, sig_child);
 	sigemptyset(&newmask);
 	sigaddset(&newmask, SIGCHLD);
+	sigaddset(&newmask, SIGTERM);
+	sigaddset(&newmask, SIGKILL);
 	pthread_sigmask(SIG_UNBLOCK, &newmask, 0L);
 
 	*pid = child;
@@ -202,7 +210,9 @@ FILE *mypopen(char **argv, pid_t * pid, int *control)
 	// make the operations on the control pipe non-blocking
 	flags = fcntl(*control, F_GETFL, 0);
 	flags |= O_NONBLOCK;
+#ifndef BSD
 	flags |= O_NDELAY;
+#endif
 	fcntl(*control, F_SETFL, flags);
 
 	return fdopen(filedesr[0], "r");
@@ -217,6 +227,10 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
     char *filename;
     char *baseurl;
     char buffer[1024];
+#ifndef X_DISPLAY_MISSING
+    char *dispName = XDisplayName(NULL);
+#endif
+    bool isRemoteDisplay = false;
 
     if (instance->threadsetup == 1)
 	return;
@@ -241,10 +255,16 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
     }
 
     if (instance->href) {
+	if (DEBUG)
+	    printf("using href for url\n");
 	snprintf(instance->td->list->url, 1024, "%s", instance->href);
     } else if (instance->fname) {
+	if (DEBUG)
+	    printf("using fname for url\n");
 	snprintf(instance->td->list->url, 1024, "%s", instance->fname);
     } else {
+	if (DEBUG)
+	    printf("using url for url\n");
 	snprintf(instance->td->list->url, 1024, "%s", instance->url);
     }
 
@@ -285,7 +305,8 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	if (filename)
 	    NPN_MemFree(filename);
     } else {
-	if (instance->nomediacache == 0) {
+	if ((instance->nomediacache == 0)
+	    && (instance->td->list->bytes > 0)) {
 	    if (strlen(instance->td->list->fname) == 0) {
 		snprintf(instance->td->list->fname, 1024, "%s",
 			 tempnam("/tmp", "mplayerplug-inXXXXXX"));
@@ -391,20 +412,38 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	// do nothing
     } else {
 	if (instance->mode == NP_EMBED) {
-	    if (instance->maintain_aspect == 1) {
-		snprintf(buffer, 1024, "-xy");
-		instance->td->argv[i++] = strdup(buffer);
-		snprintf(buffer, 1024, "%s", xval);
-		instance->td->argv[i++] = strdup(buffer);
-	    } else {
-		snprintf(buffer, 1024, "-x");
-		instance->td->argv[i++] = strdup(buffer);
-		snprintf(buffer, 1024, "%s", xval);
-		instance->td->argv[i++] = strdup(buffer);
-		snprintf(buffer, 1024, "-y");
-		instance->td->argv[i++] = strdup(buffer);
-		snprintf(buffer, 1024, "%s", yval);
-		instance->td->argv[i++] = strdup(buffer);
+#ifndef X_DISPLAY_MISSING
+	    // on remote display, XShm will be disabled.
+	    // it should not specified aspect. 
+	    if (dispName) {
+		if (strncmp(dispName, "unix:", 5) == 0)
+		    dispName += 4;
+		else if (strncmp(dispName, "localhost:", 10) == 0)
+		    dispName += 9;
+		if (*dispName != ':' || atoi(dispName + 1) >= 10) {
+		    isRemoteDisplay = true;
+		    instance->maintain_aspect = 0;
+		    if (DEBUG)
+			printf("x11 is running on remote display.\n");
+		}
+	    }
+#endif
+	    if (isRemoteDisplay == false) {
+		if (instance->maintain_aspect == 1) {
+		    snprintf(buffer, 1024, "-xy");
+		    instance->td->argv[i++] = strdup(buffer);
+		    snprintf(buffer, 1024, "%s", xval);
+		    instance->td->argv[i++] = strdup(buffer);
+		} else {
+		    snprintf(buffer, 1024, "-x");
+		    instance->td->argv[i++] = strdup(buffer);
+		    snprintf(buffer, 1024, "%s", xval);
+		    instance->td->argv[i++] = strdup(buffer);
+		    snprintf(buffer, 1024, "-y");
+		    instance->td->argv[i++] = strdup(buffer);
+		    snprintf(buffer, 1024, "%s", yval);
+		    instance->td->argv[i++] = strdup(buffer);
+		}
 	    }
 	}
     }
@@ -413,16 +452,15 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	snprintf(buffer, 1024, "-rtsp-stream-over-tcp");
 	instance->td->argv[i++] = strdup(buffer);
     }
-
-    if (instance->cachesize > 0) {
-	snprintf(buffer, 1024, "-cache");
-	instance->td->argv[i++] = strdup(buffer);
-	snprintf(buffer, 1024, "%i", instance->cachesize);
-	instance->td->argv[i++] = strdup(buffer);
-    } else {
-	snprintf(buffer, 1024, "-nocache");
-	instance->td->argv[i++] = strdup(buffer);
-    }
+//    if (instance->cachesize > 0) {
+//      snprintf(buffer, 1024, "-cache");
+//      instance->td->argv[i++] = strdup(buffer);
+//      snprintf(buffer, 1024, "%i", instance->cachesize);
+//      instance->td->argv[i++] = strdup(buffer);
+//    } else {
+//      snprintf(buffer, 1024, "-nocache");
+//      instance->td->argv[i++] = strdup(buffer);
+//    }
 
 //    if (instance->loop == 1) {
 //      snprintf(instance->td->argv[i++], 1024, "-loop");
@@ -455,6 +493,32 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	instance->td->argv[i++] = strdup(buffer);
     }
 
+    if (instance->framedrop == 1) {
+	snprintf(buffer, 1024, "-framedrop");
+	instance->td->argv[i++] = strdup(buffer);
+    }
+
+    if (instance->autosync > 0) {
+	snprintf(buffer, 1024, "-autosync");
+	instance->td->argv[i++] = strdup(buffer);
+	snprintf(buffer, 1024, "%i", instance->autosync);
+	instance->td->argv[i++] = strdup(buffer);
+    }
+
+    if (instance->mc > 0) {
+	snprintf(buffer, 1024, "-mc");
+	instance->td->argv[i++] = strdup(buffer);
+	snprintf(buffer, 1024, "%i", instance->mc);
+	instance->td->argv[i++] = strdup(buffer);
+    }
+
+    if (instance->useragent) {
+	snprintf(buffer, 1024, "-user-agent");
+	instance->td->argv[i++] = strdup(buffer);
+	snprintf(buffer, 1024, "%s", instance->useragent);
+	instance->td->argv[i++] = strdup(buffer);
+    }
+
     snprintf(buffer, 1024, "-osdlevel");
     instance->td->argv[i++] = strdup(buffer);
     snprintf(buffer, 1024, "%i", instance->osdlevel);
@@ -463,6 +527,15 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
     /* some extra flags for mplayer */
     snprintf(buffer, 1024, "-nojoystick");	/* annoying... */
     instance->td->argv[i++] = strdup(buffer);
+    if (instance->noconsolecontrols) {
+	snprintf(buffer, 1024, "-noconsolecontrols");
+	instance->td->argv[i++] = strdup(buffer);
+    }
+
+    if (instance->cookies) {
+	snprintf(buffer, 1024, "-cookies");
+	instance->td->argv[i++] = strdup(buffer);
+    }
 //    snprintf(buffer, 1024, "-nofs");  /* no full screen */
 //    instance->td->argv[i++] = strdup(buffer);
     snprintf(buffer, 1024, "-slave");	/* slave mode */
@@ -572,7 +645,8 @@ void refresh_frame(char *buffer, _ThreadData * td, Node * node)
 //return true if we should try to play this again immediately, false if not 
 
 int playNode(ThreadData * local_td, Node * local_list, char *local_url,
-	     int local_mmsstream, int *usefps, int *nomouseinput)
+	     int local_mmsstream, int *usefps, int *nomouseinput,
+	     int *maybeplaylist)
 {
 
     char buffer[1024];
@@ -591,7 +665,9 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
     int zerocfbytes_count = 0;
     char url_copy[1024];
     int c;
-
+#ifdef GTK_ENABLED
+    int fsupdated = 0;
+#endif
 #define TRYAGAIN_FALSE 0
 #define TRYAGAIN_TRUE 1
 #define TRYAGAIN_FALLBACK 2
@@ -621,6 +697,9 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 
     while (1) {
 	pthread_testcancel();
+	if (local_td->instance->player == NULL)
+	    break;
+
 	if (feof(local_td->instance->player)) {
 	    pthread_testcancel();
 	    break;
@@ -667,7 +746,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    } while (((unsigned char) c != '\n'));
 	    if (buffer[0] == '\0')
 		continue;
-	    else
+	    else if (i < 1024)	// make sure we don't access out of bounds
 		buffer[i] = '\0';	// make sure we NULL terminate the string
 	}
 	pthread_testcancel();
@@ -742,8 +821,8 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    }
 	}
 
+	pthread_testcancel();
 	if (strstr(buffer, "Starting") != NULL) {
-	    pthread_testcancel();
 	    strlcpy(message, "Playing ", 1024);
 	    strlcat(message, local_url, 1000);
 #ifdef X_ENABLED
@@ -756,6 +835,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 #endif
 	}
 
+	pthread_testcancel();
 	if (strstr(buffer, "VO:") != NULL) {
 	    if (local_td->instance->mode == NP_EMBED
 		&& local_td->instance->noembed == 0)
@@ -781,6 +861,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 		    g_idle_add(gtkgui_draw, local_td->instance);
 #endif
 		}
+
 		if (DEBUG)
 		    printf
 			("----player thread: panel height in thread = %i\n",
@@ -809,6 +890,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 
 	}
 
+	pthread_testcancel();
 	if (strstr(buffer, "Video: no video") != NULL) {
 	    if (local_td->instance->panel_height == 0)
 		local_td->instance->panel_height = 32;
@@ -820,6 +902,8 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    g_idle_add(gtkgui_draw, local_td->instance);
 #endif
 	}
+
+	pthread_testcancel();
 	// mplayer answer back messages
 	if (strstr(buffer, "ANS_LENGTH") != 0) {
 	    msg = strstr(buffer, "ANS_LENGTH");
@@ -830,6 +914,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 		       local_td->instance->mediaLength);
 	}
 
+	pthread_testcancel();
 	if (strstr(buffer, "ANS_PERCENT_POSITION") != 0) {
 	    msg = strstr(buffer, "ANS_PERCENT_POSITION");
 	    sscanf(msg, "ANS_PERCENT_POSITION=%i",
@@ -847,19 +932,24 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    g_idle_add(gtkgui_drawMediaProgress, local_td->instance);
 #endif
 	}
+
+	pthread_testcancel();
 	// this is only needed for the retry code
-	if (strstr(buffer, "\rA: ") != 0 || strstr(buffer, "\rV: ") != 0) {
+	if (strstr(buffer, "\rA:") != 0 || strstr(buffer, "\rV:") != 0) {
 	    //audio or video is playing
-	    msg = strstr(buffer, "\rA: ");
+	    msg = strstr(buffer, "\rA:");
 	    if (msg != NULL) {
-		sscanf(msg, "\rA: %f", &(local_td->instance->mediaTime));
+		sscanf(msg, "\rA:%f", &(local_td->instance->mediaTime));
 		if (DEBUG > 1)
 		    printf("mediaTime = %f\n",
 			   local_td->instance->mediaTime);
+#ifdef GTK_ENABLED
+		g_idle_add(gtkgui_draw, local_td->instance);
+#endif
 	    } else {
-		msg = strstr(buffer, "\rV: ");
+		msg = strstr(buffer, "\rV:");
 		if (msg != NULL) {
-		    sscanf(msg, "\rV: %f",
+		    sscanf(msg, "\rV:%f",
 			   &(local_td->instance->mediaTime));
 		    if (DEBUG > 1)
 			printf("mediaTime = %f\n",
@@ -872,7 +962,10 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 					    control_mutex));
 	    pthread_mutex_lock(&(local_td->instance->control_mutex));
 
-	    local_td->instance->js_state = JS_STATE_PLAYING;
+	    if ((local_td->instance->js_state != JS_STATE_STOPPED)
+		&& (local_td->instance->js_state != JS_STATE_PAUSED))
+		local_td->instance->js_state = JS_STATE_PLAYING;
+
 	    pthread_mutex_unlock(&(local_td->instance->control_mutex));
 	    pthread_cleanup_pop(0);
 	    if (local_td->instance->paused == 0) {
@@ -884,10 +977,18 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 		sendCommand(local_td->instance, "get_percent_pos");
 		pthread_mutex_unlock(&(local_td->instance->control_mutex));
 		pthread_cleanup_pop(0);
+#ifdef GTK_ENABLED
+		if (fsupdated == 0) {
+		    g_idle_add(gtkgui_updatefullscreen,
+			       local_td->instance);
+		    fsupdated = 1;
+		}
+#endif
 	    }
 	    tryagain = TRYAGAIN_FALSE;
 	}
 
+	pthread_testcancel();
 	if (strstr(buffer, "Connect") != NULL) {
 	    snprintf(message, 1024, "%s", buffer);
 #ifdef X_ENABLED
@@ -900,6 +1001,41 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 #endif
 	}
 
+	pthread_testcancel();
+	if (strstr(buffer, "No stream found") != NULL) {
+	    snprintf(message, 1024, "%s", buffer);
+#ifdef X_ENABLED
+	    DrawUI(local_td->instance->widget,
+		   local_td->instance, message, 0, -1);
+#endif
+#ifdef GTK_ENABLED
+	    snprintf(local_td->instance->lastmessage, 1024, "%s", message);
+	    g_idle_add(gtkgui_message, local_td->instance);
+#endif
+	}
+
+
+	pthread_testcancel();
+	if (strstr(buffer, "Example: mplayer -playlist ") != NULL) {
+	    if (*maybeplaylist == 0) {
+		tryagain = TRYAGAIN_TRUE;
+		*maybeplaylist = 1;
+	    } else
+		tryagain = TRYAGAIN_FALSE;
+	    break;
+	}
+
+	pthread_testcancel();
+	if (strstr(buffer, "Stream not seekable") != NULL) {
+	    if (*maybeplaylist == 0) {
+		tryagain = TRYAGAIN_TRUE;
+		*maybeplaylist = 1;
+	    } else
+		tryagain = TRYAGAIN_FALSE;
+	    break;
+	}
+
+	pthread_testcancel();
 	if (strstr(buffer, "Error while decoding") != NULL) {
 	    if (!isMms(local_url)) {
 		if (DEBUG)
@@ -917,6 +1053,7 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    }
 	}
 
+	pthread_testcancel();
 	if (strstr(buffer, "Quit") != NULL) {
 	    if (DEBUG)
 		printf("----player thread: breaking read loop - Quit\n");
@@ -924,6 +1061,8 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    tryagain = TRYAGAIN_FALSE;
 	    break;
 	}
+
+	pthread_testcancel();
 	// detect Quicktime file with old codec
 	if (strstr(buffer, "MOV: missing header (moov/cmov) chunk!") !=
 	    NULL) {
@@ -952,18 +1091,28 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    break;
 	}
 
+	pthread_testcancel();
 	if (strstr(buffer, "FPS not specified") != NULL) {
 	    tryagain = TRYAGAIN_TRUE;
 	    *usefps = 1;
 	    break;
 	}
 
+	pthread_testcancel();
 	if (strstr(buffer, "nomouseinput") != NULL) {
-	    *nomouseinput = 1;
+	    *nomouseinput = 0;
 	    tryagain = TRYAGAIN_TRUE;
 	    break;
 	}
+// Possible work around for cinema now site     
+//      if (strstr(buffer, "ASF file format detected") != NULL) {
+//          tryagain = TRYAGAIN_TRUE;
+//          local_list->playlist = 1;
+//          break;
+//      }
 
+
+	pthread_testcancel();
 	if (strstr(buffer, "Exiting") != NULL) {
 	    if (DEBUG)
 		printf
@@ -977,6 +1126,8 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    tryagain = TRYAGAIN_FALSE;
 	    break;
 	}
+
+	pthread_testcancel();
 	// break out if mplayer crashes
 	if (strstr(buffer, "interrupted") != NULL) {
 	    if (DEBUG)
@@ -985,12 +1136,14 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 	    break;
 	}
 
+	pthread_testcancel();
 	if (strstr(buffer, "explicit kill") != NULL) {
 	    if (DEBUG)
 		printf("----player thread: breaking read loop - killed\n");
 	    break;
 	}
 
+	pthread_testcancel();
 	if (strstr(buffer, "everything done") != NULL) {
 	    if (DEBUG)
 		printf
@@ -1000,21 +1153,8 @@ int playNode(ThreadData * local_td, Node * local_list, char *local_url,
 
 
 	assert(local_td->instance->player != NULL);
-/*
-	//may not be completely safe
-	pthread_mutex_lock(&(local_td->instance->playlist_mutex));
-	if (local_td->instance->cancelled == 1) {
-	    pthread_mutex_unlock(&(local_td->instance->playlist_mutex));
-	    if (DEBUG)
-		printf
-		    ("----player thread: breaking read loop - cancelled\n");
-	    tryagain = TRYAGAIN_FALSE;
-	    break;
-	}
-	pthread_mutex_unlock(&(local_td->instance->playlist_mutex));
-*/
-    }
 
+    }				// end of READ LOOP
 
     if (DEBUG) {
 	printf("----player thread: tryagain = %d\n", tryagain);
@@ -1056,6 +1196,7 @@ void *playPlaylist(void *td)
     char cmd[2048];
     char message[1024];
     char mmsplaylist[1024];
+    char buffer[1024];		// scratch pad
 
     int argc = 0, base_argc = 0;
     ThreadData *local_td;
@@ -1063,11 +1204,14 @@ void *playPlaylist(void *td)
     char local_url[1024];
     int local_mmsstream;
     int i;
+    int loop_single = 0;
+    int loop_count;
     Node *node;
 
     int tryagain;
     int usefps;
     int nomouseinput;
+    int maybeplaylist;
     int listempty;
 
     local_td = (ThreadData *) td;
@@ -1095,21 +1239,44 @@ void *playPlaylist(void *td)
 
 
     if (local_td->instance->state < STATE_STARTED_PLAYER) {
+	// wait for playlist_complete_cond to be signalled
+	// this should happen when we have completed getting all the playlist elements
 	pthread_cond_wait(&(local_td->instance->playlist_complete_cond),
 			  &(local_td->instance->playlist_cond_mutex));
+	pthread_mutex_unlock(&(local_td->instance->playlist_cond_mutex));
 
-	assert(local_td != NULL);
-	assert(local_td->instance != NULL);
-
-	local_td->instance->state = STATE_STARTED_PLAYER;
-	if (DEBUG)
-	    printf
-		("---player thread: got wakeup signal, js_state = %d, state = %d\n",
-		 local_td->instance->js_state, local_td->instance->state);
+	pthread_testcancel();
+	sleep(1);
+	// playlist_cond_mutex should be locked now
+	if (local_td != NULL) {
+	    if (DEBUG > 1)
+		printf("local_td = %p\n", local_td);
+	    if (local_td->instance != NULL) {
+		if (DEBUG > 1)
+		    printf("local_td->instance = %p\n",
+			   local_td->instance);
 
+		pthread_testcancel();
+		local_td->instance->state = STATE_STARTED_PLAYER;
+		if (DEBUG)
+		    printf
+			("---player thread: got wakeup signal, js_state = %d, state = %d\n",
+			 local_td->instance->js_state,
+			 local_td->instance->state);
+	    } else {
+		if (DEBUG)
+		    printf
+			("We were signalled to start but local_td->instance is NULL, should not happen\n");
+		pthread_exit(0);
+	    }
+	} else {
+	    if (DEBUG)
+		printf
+		    ("We were signalled to start but local_td is NULL, should not happen\n");
+	    pthread_exit(0);
+	}
     }
 
-    pthread_mutex_unlock(&(local_td->instance->playlist_cond_mutex));
     pthread_cleanup_pop(0);
 
     if (DEBUG)
@@ -1162,13 +1329,39 @@ void *playPlaylist(void *td)
     if (DEBUG) {
 	while (local_list != NULL) {
 	    printf
-		("----player thread: local_list =  %p\nPL URL: %s\nplay = %i, cancelled= %i\nnext= %p\n",
+		("----player thread:\nlocal_list =  %p\nPL URL: %s\nplay = %i, cancelled= %i\nnext= %p\n",
 		 local_list,
 		 local_list->url, local_list->play,
 		 local_list->cancelled, local_list->next);
 	    local_list = local_list->next;
 	}
     }
+    // count playable items in the list, if playable =1 and loop = 1 then add a couple of args
+    if (local_td->instance->loop >= 0) {
+	i = 0;
+	local_list = local_td->list;
+	while (local_list != NULL) {
+	    if (local_list->play)
+		i++;
+	    local_list = local_list->next;
+	}
+	if (DEBUG)
+	    printf
+		("----player thread:\nFound %i files to play and loop is true\n",
+		 i);
+
+	// if the play count is exactly 1 then tell mplayer to loop continuously and to use the same frame buffer (should eliminate blinking)
+	if (i == 1) {
+	    loop_single = 1;
+	    local_td->argv[argc++] = strdup("-loop");
+	    snprintf(buffer, 32, "%i", local_td->instance->loop);
+	    local_td->argv[argc++] = strdup(buffer);
+	    local_td->argv[argc++] = strdup("-fixed-vo");
+	    base_argc = argc;
+	}
+    }
+
+    loop_count = local_td->instance->loop;
 
     while (1) {
 	if (DEBUG)
@@ -1194,8 +1387,13 @@ void *playPlaylist(void *td)
 	}
 
 	if (local_list == NULL) {
-	    //nothing to play: will exit
-	    if (local_td->instance->loop) {
+	    //nothing to play: will exit, unless we have loop
+	    // the -- is to countdown the loop
+	    if (local_td->instance->loop != 0)
+		loop_count--;
+
+	    // we're gonna reset the playlist, because we are going to play it again
+	    if ((loop_count >= 0) && (loop_single == 0)) {
 		local_list = local_td->list;
 		while (local_list != NULL) {
 		    if (local_list->play)
@@ -1203,7 +1401,7 @@ void *playPlaylist(void *td)
 		    local_list = local_list->next;
 		}
 
-		// rescan again
+		// rescan again, find the first playable item from the top of the list.
 		local_list = local_td->list;
 		while (local_list != NULL) {
 		    if (local_list->play && !(local_list->played)) {
@@ -1240,7 +1438,8 @@ void *playPlaylist(void *td)
 	pthread_testcancel();
 
 	usefps = 0;
-	nomouseinput = 0;
+	nomouseinput = local_td->instance->nomouseinput;
+	maybeplaylist = 0;
 
 	do {
 
@@ -1283,14 +1482,20 @@ void *playPlaylist(void *td)
 		usefps = 0;
 	    }
 
-	    if (nomouseinput == 0) {
+	    if (nomouseinput) {
 		local_td->argv[argc++] = strdup("-nomouseinput");
-	    } else {
-		nomouseinput = 1;
+//          } else {
+//              nomouseinput = 0;
+	    }
+
+	    if (local_td->instance->starttime > 0) {
+		local_td->argv[argc++] = strdup("-ss");
+		snprintf(message, 1024, "%li",
+			 local_td->instance->starttime);
+		local_td->argv[argc++] = strdup(message);
+		local_td->instance->starttime = 0;
 	    }
 
-	    if (local_list->playlist == 1)
-		local_td->argv[argc++] = strdup("-playlist");
 
 	    if (local_td->instance != NULL)
 		local_td->instance->mmsstream = local_list->mmsstream;
@@ -1298,6 +1503,11 @@ void *playPlaylist(void *td)
 #ifdef GTK_ENABLED
 		g_idle_add(gtkgui_updatebuttons, local_td->instance);
 #endif
+		local_td->argv[argc++] = strdup("-cache");
+		snprintf(message, 1024, "%i",
+			 local_td->instance->cachesize);
+		local_td->argv[argc++] = strdup(message);
+
 		local_td->argv[argc++] = strdup(local_url);
 		if (local_td->instance->keep_download) {
 		    snprintf(mmsplaylist, sizeof(mmsplaylist),
@@ -1313,10 +1523,14 @@ void *playPlaylist(void *td)
 		if (strlen(local_list->fname) == 0) {
 		    local_td->argv[argc++] = strdup(local_url);
 		} else {
+		    local_td->argv[argc++] = strdup("-nocache");
 		    local_td->argv[argc++] = strdup(local_list->fname);
 		}
 	    }
 
+	    if (local_list->playlist == 1 || maybeplaylist != 0)
+		local_td->argv[argc++] = strdup("-playlist");
+
 	    if (DEBUG) {
 		printf("----player thread: URL: %s\n", local_url);
 	    }
@@ -1334,8 +1548,8 @@ void *playPlaylist(void *td)
 	    pthread_mutex_lock(&(local_td->instance->control_mutex));
 
 	    local_td->instance->player =
-		mypopen(local_td->argv, &local_td->instance->pid,
-			&local_td->instance->control);
+		mypopen(local_td->argv, &(local_td->instance->pid),
+			&(local_td->instance->control));
 	    if (local_td->instance->player != NULL) {
 		local_td->instance->js_state = JS_STATE_PLAYING;
 	    }
@@ -1354,7 +1568,8 @@ void *playPlaylist(void *td)
 
 		tryagain =
 		    playNode(local_td, local_list, local_url,
-			     local_mmsstream, &usefps, &nomouseinput);
+			     local_mmsstream, &usefps, &nomouseinput,
+			     &maybeplaylist);
 
 		if (DEBUG)
 		    printf
@@ -1445,7 +1660,7 @@ void *playPlaylist(void *td)
     if (DEBUG) {
 	printf("----player thread: callbacks complete\n");
     }
-
+    // free all the data (command line parameters) that is used to start mplayer
     pthread_mutex_lock(&(local_td->instance->control_mutex));
     local_td->instance->js_state = JS_STATE_UNDEFINED;
     argc = base_argc;
@@ -1456,6 +1671,11 @@ void *playPlaylist(void *td)
     }
     pthread_mutex_unlock(&(local_td->instance->control_mutex));
 
+    // since thread is dead, reset thread state variables.
+    local_td->instance->threadsetup = 0;
+    local_td->instance->threadlaunched = 0;
+    local_td->instance->threadsignaled = 0;
+
     //once we get here, the thread is only allowed to access stack
     // variables and then exit (since another player thread may have
     // been started at this time).    
@@ -1464,7 +1684,6 @@ void *playPlaylist(void *td)
 	printf("----player thread: normal exit\n");
     }
 
-
     pthread_exit(0);
 
     return NULL;
