@@ -15,6 +15,7 @@ extern int DEBUG;
 #endif
 
 #ifdef GTK_ENABLED
+#include <gdk/gdkkeysyms.h>
 
 // 16 x 12 button pixmaps
 #include "../pixmaps/play_down_small.xpm"
@@ -462,7 +463,7 @@ void play_callback(GtkWidget * widget, GdkEventExpose * event,
 
     if (instance == NULL)
 	return;
-    if (instance->panel_drawn == 0)
+    if (instance->panel_drawn == 0 && instance->controlsvisible == 1)
 	return;
     if (instance->controlsvisible == 1) {
 	if (instance->panel_height > 16)
@@ -497,15 +498,19 @@ void play_callback(GtkWidget * widget, GdkEventExpose * event,
 			  instance->image_pause);
 	gtk_container_add(GTK_CONTAINER(instance->stop_event_box),
 			  instance->image_stop);
-	gtk_widget_show(instance->image_play);
-	gtk_widget_show(instance->image_pause);
-	gtk_widget_show(instance->image_stop);
-
-	gtk_widget_show(instance->play_event_box);
-	gtk_widget_show(instance->pause_event_box);
-	gtk_widget_show(instance->stop_event_box);
+	if (instance->showbuttons) {
+	    gtk_widget_show(instance->image_play);
+	    gtk_widget_show(instance->image_pause);
+	    gtk_widget_show(instance->image_stop);
 
+	    gtk_widget_show(instance->play_event_box);
+	    gtk_widget_show(instance->pause_event_box);
+	    gtk_widget_show(instance->stop_event_box);
+	}
 	gtk_widget_show(instance->fixed_container);
+	if (instance->drawing_area != NULL)
+	    gtk_widget_show(GTK_WIDGET(instance->drawing_area));
+
 	gdk_flush();
     }
     if (widget != NULL)
@@ -520,7 +525,7 @@ void pause_callback(GtkWidget * widget, GdkEventExpose * event,
 
     if (instance == NULL)
 	return;
-    if (instance->panel_drawn == 0)
+    if (instance->panel_drawn == 0 && instance->controlsvisible == 1)
 	return;
     if (instance->controlsvisible == 1) {
 	if (instance->panel_height > 16)
@@ -557,15 +562,18 @@ void pause_callback(GtkWidget * widget, GdkEventExpose * event,
 	gtk_container_add(GTK_CONTAINER(instance->stop_event_box),
 			  instance->image_stop);
 
-	gtk_widget_show(instance->image_play);
-	gtk_widget_show(instance->image_pause);
-	gtk_widget_show(instance->image_stop);
-
-	gtk_widget_show(instance->play_event_box);
-	gtk_widget_show(instance->pause_event_box);
-	gtk_widget_show(instance->stop_event_box);
+	if (instance->showbuttons) {
+	    gtk_widget_show(instance->image_play);
+	    gtk_widget_show(instance->image_pause);
+	    gtk_widget_show(instance->image_stop);
 
+	    gtk_widget_show(instance->play_event_box);
+	    gtk_widget_show(instance->pause_event_box);
+	    gtk_widget_show(instance->stop_event_box);
+	}
 	gtk_widget_show(instance->fixed_container);
+	if (instance->drawing_area != NULL)
+	    gtk_widget_show(GTK_WIDGET(instance->drawing_area));
 	gdk_flush();
     }
     if (widget != NULL)
@@ -582,7 +590,7 @@ void stop_callback(GtkWidget * widget, GdkEventExpose * event,
 	return;
     if (instance->mInitialized == FALSE)
 	return;
-    if (instance->panel_drawn == 0)
+    if (instance->panel_drawn == 0 && instance->controlsvisible == 1)
 	return;
     if (instance->controlsvisible == 1) {
 	if (instance->panel_height > 16)
@@ -617,16 +625,23 @@ void stop_callback(GtkWidget * widget, GdkEventExpose * event,
 			  instance->image_pause);
 	gtk_container_add(GTK_CONTAINER(instance->stop_event_box),
 			  instance->image_stop);
-	gtk_widget_show(instance->image_play);
-	gtk_widget_show(instance->image_pause);
-	gtk_widget_show(instance->image_stop);
-
-	gtk_widget_show(instance->play_event_box);
-	gtk_widget_show(instance->pause_event_box);
-	gtk_widget_show(instance->stop_event_box);
+	if (instance->showbuttons) {
+	    gtk_widget_show(instance->image_play);
+	    gtk_widget_show(instance->image_pause);
+	    gtk_widget_show(instance->image_stop);
 
+	    gtk_widget_show(instance->play_event_box);
+	    gtk_widget_show(instance->pause_event_box);
+	    gtk_widget_show(instance->stop_event_box);
+	}
 	gtk_widget_show(instance->fixed_container);
 	gtk_widget_hide(GTK_WIDGET(instance->progress_bar));
+	if (instance->drawing_area != NULL) {
+	    gtk_widget_hide(GTK_WIDGET(instance->drawing_area));
+	    snprintf(instance->lastmessage, 1024, "Stopped");
+	    g_idle_add(gtkgui_message, instance);
+	    gtk_widget_hide(GTK_WIDGET(instance->mediaprogress_bar));
+	}
 	gdk_flush();
     }
     if (widget != NULL)
@@ -639,7 +654,7 @@ void ff_callback(GtkWidget * widget, GdkEventExpose * event,
 {
     if (instance == NULL)
 	return;
-    if (instance->panel_drawn == 0)
+    if (instance->panel_drawn == 0 && instance->controlsvisible == 1)
 	return;
     if (instance->controlsvisible == 1) {
 	if (instance->panel_height > 16)
@@ -657,8 +672,10 @@ void ff_callback(GtkWidget * widget, GdkEventExpose * event,
 
 	gtk_container_add(GTK_CONTAINER(instance->ff_event_box),
 			  instance->image_ff);
-	gtk_widget_show(instance->image_ff);
-	gtk_widget_show(instance->ff_event_box);
+	if (instance->showbuttons) {
+	    gtk_widget_show(instance->image_ff);
+	    gtk_widget_show(instance->ff_event_box);
+	}
 	gdk_flush();
 	instance->FastForward();
 	usleep(500);
@@ -675,8 +692,10 @@ void ff_callback(GtkWidget * widget, GdkEventExpose * event,
 
 	gtk_container_add(GTK_CONTAINER(instance->ff_event_box),
 			  instance->image_ff);
-	gtk_widget_show(instance->image_ff);
-	gtk_widget_show(instance->ff_event_box);
+	if (instance->showbuttons) {
+	    gtk_widget_show(instance->image_ff);
+	    gtk_widget_show(instance->ff_event_box);
+	}
     }
 
 }
@@ -686,7 +705,7 @@ void rew_callback(GtkWidget * widget, GdkEventExpose * event,
 {
     if (instance == NULL)
 	return;
-    if (instance->panel_drawn == 0)
+    if (instance->panel_drawn == 0 && instance->controlsvisible == 1)
 	return;
     if (instance->controlsvisible == 1) {
 
@@ -706,8 +725,10 @@ void rew_callback(GtkWidget * widget, GdkEventExpose * event,
 
 	gtk_container_add(GTK_CONTAINER(instance->rew_event_box),
 			  instance->image_rew);
-	gtk_widget_show(instance->image_rew);
-	gtk_widget_show(instance->rew_event_box);
+	if (instance->showbuttons) {
+	    gtk_widget_show(instance->image_rew);
+	    gtk_widget_show(instance->rew_event_box);
+	}
 	gdk_flush();
 	instance->FastReverse();
 	usleep(500);
@@ -724,8 +745,10 @@ void rew_callback(GtkWidget * widget, GdkEventExpose * event,
 
 	gtk_container_add(GTK_CONTAINER(instance->rew_event_box),
 			  instance->image_rew);
-	gtk_widget_show(instance->image_rew);
-	gtk_widget_show(instance->rew_event_box);
+	if (instance->showbuttons) {
+	    gtk_widget_show(instance->image_rew);
+	    gtk_widget_show(instance->rew_event_box);
+	}
     }
 
 }
@@ -734,20 +757,23 @@ void fs_callback(GtkWidget * widget, GdkEventExpose * event,
 		 nsPluginInstance * instance)
 {
     if (DEBUG)
-	printf("pause clickd\n");
+	printf("fs_callback clicked\n");
     if (instance == NULL)
 	return;
 
-    if (instance->panel_drawn == 0)
+    if (instance->panel_drawn == 0 && instance->controlsvisible == 1)
 	return;
+
     if (instance->controlsvisible == 1) {
 	if (instance->panel_height > 16)
 	    instance->panel_height = 16;
 
 	gtk_container_remove(GTK_CONTAINER(instance->fs_event_box),
 			     instance->image_fs);
+	if (widget != NULL)
+	    instance->SetFullscreen(!instance->fullscreen);
 
-	if (instance->fullscreen) {
+	if (instance->fullscreen == 0) {
 #ifdef GTK2_ENABLED
 	    instance->image_fs =
 		gtk_image_new_from_pixbuf(instance->pb_sm_fs_up);
@@ -771,16 +797,54 @@ void fs_callback(GtkWidget * widget, GdkEventExpose * event,
 	gtk_container_add(GTK_CONTAINER(instance->fs_event_box),
 			  instance->image_fs);
 
-	gtk_widget_show(instance->image_fs);
-	gtk_widget_show(instance->fs_event_box);
+	if (instance->showfsbutton && instance->showbuttons) {
+	    gtk_widget_show(instance->image_fs);
+	    gtk_widget_show(instance->fs_event_box);
+	}
 	gtk_widget_show(instance->fixed_container);
 	gdk_flush();
     }
-    if (widget != NULL)
-	instance->SetFullscreen(!instance->fullscreen);
 }
 
+gboolean keyboard_callback(GtkWidget * widget, GdkEventKey * event,
+			   nsPluginInstance * instance)
+{
+    if (DEBUG)
+	printf("In keyboard_callback with %i\n", event->keyval);
 
+    if ((event->keyval == GDK_P) || (event->keyval == GDK_p)
+	|| (event->keyval == GDK_space)) {
+	if (instance->paused) {
+	    play_callback(widget, (GdkEventExpose *) NULL, instance);
+	} else {
+	    pause_callback(widget, (GdkEventExpose *) NULL, instance);
+	}
+	return TRUE;
+    }
+
+    if ((event->keyval == GDK_S) || (event->keyval == GDK_s)) {
+	stop_callback(widget, (GdkEventExpose *) NULL, instance);
+	return TRUE;
+    }
+
+    if ((event->keyval == GDK_less) || (event->keyval == GDK_comma)) {
+	rew_callback(widget, (GdkEventExpose *) NULL, instance);
+	return TRUE;
+    }
+
+    if ((event->keyval == GDK_greater) || (event->keyval == GDK_period)) {
+	ff_callback(widget, (GdkEventExpose *) NULL, instance);
+	return TRUE;
+    }
+
+    if ((event->keyval == GDK_F) || (event->keyval == GDK_f)) {
+	fs_callback(widget, (GdkEventExpose *) NULL, instance);
+	return TRUE;
+    }
+
+
+    return FALSE;
+}
 
 gboolean gtkgui_draw(void *data)
 {
@@ -795,7 +859,7 @@ gboolean gtkgui_draw(void *data)
     if (instance == NULL)
 	return FALSE;
     if (DEBUG > 1)
-	printf("panel_height = %i\nDrawn = %i", instance->panel_height,
+	printf("panel_height = %i\nDrawn = %i\n", instance->panel_height,
 	       instance->panel_drawn);
 
     if (instance->mode == NP_EMBED && instance->window == 0)
@@ -816,22 +880,28 @@ gboolean gtkgui_draw(void *data)
 	    instance->panel_height = 16;
 	if (instance->panel_height < 0)
 	    instance->panel_height = 0;
-	if (DEBUG > 1)
+	if (DEBUG > 1) {
 	    printf("drawing panel\n height = %i panel= %i\n",
 		   win_height, instance->panel_height);
+	    printf("showbuttons = %i\nshowcontrols = %i\n",
+		   instance->showbuttons, instance->showcontrols);
+	}
 	if (instance->panel_height > 0 && instance->showcontrols == 1) {
 #ifdef GTK2_ENABLED
 	    if (instance->autostart) {
 		instance->image_play =
 		    gtk_image_new_from_pixbuf(instance->pb_sm_play_down);
+		instance->image_stop =
+		    gtk_image_new_from_pixbuf(instance->pb_sm_stop_up);
 	    } else {
 		instance->image_play =
 		    gtk_image_new_from_pixbuf(instance->pb_sm_play_up);
+		instance->image_stop =
+		    gtk_image_new_from_pixbuf(instance->pb_sm_stop_down);
 	    }
 	    instance->image_pause =
 		gtk_image_new_from_pixbuf(instance->pb_sm_pause_up);
-	    instance->image_stop =
-		gtk_image_new_from_pixbuf(instance->pb_sm_stop_up);
+
 	    instance->image_ff =
 		gtk_image_new_from_pixbuf(instance->pb_sm_ff_up);
 	    instance->image_rew =
@@ -843,14 +913,18 @@ gboolean gtkgui_draw(void *data)
 	    if (instance->autostart) {
 		instance->image_play =
 		    gtk_pixmap_new(instance->pb_sm_play_down, NULL);
+		instance->image_stop =
+		    gtk_pixmap_new(instance->pb_sm_stop_up, NULL);
+
 	    } else {
 		instance->image_play =
 		    gtk_pixmap_new(instance->pb_sm_play_up, NULL);
+		instance->image_stop =
+		    gtk_pixmap_new(instance->pb_sm_stop_down, NULL);
+
 	    }
 	    instance->image_pause =
 		gtk_pixmap_new(instance->pb_sm_pause_up, NULL);
-	    instance->image_stop =
-		gtk_pixmap_new(instance->pb_sm_stop_up, NULL);
 	    instance->image_ff =
 		gtk_pixmap_new(instance->pb_sm_ff_up, NULL);
 	    instance->image_rew =
@@ -861,9 +935,20 @@ gboolean gtkgui_draw(void *data)
 	    width = SMALL_BUTTON_WIDTH;
 	    height = SMALL_BUTTON_HEIGHT;
 
+#ifdef GTK2_ENABLED
+	    g_signal_connect(G_OBJECT(instance->gtkwidget),
+			     "key_press_event",
+			     G_CALLBACK(keyboard_callback), instance);
+#endif
+#ifdef GTK1_ENABLED
+	    gtk_signal_connect(GTK_OBJECT(instance->gtkwidget),
+			       "key-press-event",
+			       GTK_SIGNAL_FUNC(keyboard_callback),
+			       instance);
+#endif
 	    multiplier = 0;
 	    // rewind
-	    if (instance->mmsstream == 0) {
+	    if (instance->mmsstream == 0 && win_width > 126) {
 		instance->rew_event_box = gtk_event_box_new();
 		gtk_widget_set_events(instance->rew_event_box,
 				      GDK_BUTTON_PRESS_MASK);
@@ -883,8 +968,10 @@ gboolean gtkgui_draw(void *data)
 		gtk_fixed_put(GTK_FIXED(instance->fixed_container),
 			      instance->rew_event_box,
 			      width * multiplier++, win_height - height);
-		gtk_widget_show(instance->image_rew);
-		gtk_widget_show(instance->rew_event_box);
+		if (instance->showbuttons) {
+		    gtk_widget_show(instance->image_rew);
+		    gtk_widget_show(instance->rew_event_box);
+		}
 	    }
 	    // play
 	    instance->play_event_box = gtk_event_box_new();
@@ -905,9 +992,10 @@ gboolean gtkgui_draw(void *data)
 	    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
 			  instance->play_event_box, width * multiplier++,
 			  win_height - height);
-	    gtk_widget_show(instance->image_play);
-	    gtk_widget_show(instance->play_event_box);
-
+	    if (instance->showbuttons) {
+		gtk_widget_show(instance->image_play);
+		gtk_widget_show(instance->play_event_box);
+	    }
 	    // pause
 	    instance->pause_event_box = gtk_event_box_new();
 	    gtk_widget_set_events(instance->pause_event_box,
@@ -924,12 +1012,16 @@ gboolean gtkgui_draw(void *data)
 #endif
 	    gtk_container_add(GTK_CONTAINER(instance->pause_event_box),
 			      instance->image_pause);
-	    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
-			  instance->pause_event_box, width * multiplier++,
-			  win_height - height);
-	    gtk_widget_show(instance->image_pause);
-	    gtk_widget_show(instance->pause_event_box);
+	    if (win_width > 126) {
 
+		gtk_fixed_put(GTK_FIXED(instance->fixed_container),
+			      instance->pause_event_box,
+			      width * multiplier++, win_height - height);
+		if (instance->showbuttons) {
+		    gtk_widget_show(instance->image_pause);
+		    gtk_widget_show(instance->pause_event_box);
+		}
+	    }
 	    // stop
 	    instance->stop_event_box = gtk_event_box_new();
 	    gtk_widget_set_events(instance->stop_event_box,
@@ -949,11 +1041,12 @@ gboolean gtkgui_draw(void *data)
 	    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
 			  instance->stop_event_box, width * multiplier++,
 			  win_height - height);
-	    gtk_widget_show(instance->image_stop);
-	    gtk_widget_show(instance->stop_event_box);
-
+	    if (instance->showbuttons) {
+		gtk_widget_show(instance->image_stop);
+		gtk_widget_show(instance->stop_event_box);
+	    }
 	    // fastforward
-	    if (instance->mmsstream == 0) {
+	    if (instance->mmsstream == 0 && win_width > 126) {
 		instance->ff_event_box = gtk_event_box_new();
 		gtk_widget_set_events(instance->ff_event_box,
 				      GDK_BUTTON_PRESS_MASK);
@@ -972,8 +1065,10 @@ gboolean gtkgui_draw(void *data)
 		gtk_fixed_put(GTK_FIXED(instance->fixed_container),
 			      instance->ff_event_box, width * multiplier++,
 			      win_height - height);
-		gtk_widget_show(instance->image_ff);
-		gtk_widget_show(instance->ff_event_box);
+		if (instance->showbuttons) {
+		    gtk_widget_show(instance->image_ff);
+		    gtk_widget_show(instance->ff_event_box);
+		}
 	    }
 
 	    if ((int) (width * multiplier + 10) <
@@ -1010,12 +1105,15 @@ gboolean gtkgui_draw(void *data)
 #endif
 	    gtk_container_add(GTK_CONTAINER(instance->fs_event_box),
 			      instance->image_fs);
-	    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
-			  instance->fs_event_box,
-			  win_width - width, win_height - height);
-	    gtk_widget_show(instance->image_fs);
-	    gtk_widget_show(instance->fs_event_box);
-
+	    if (win_width > 126) {
+		gtk_fixed_put(GTK_FIXED(instance->fixed_container),
+			      instance->fs_event_box,
+			      win_width - width, win_height - height);
+		if (instance->showfsbutton && instance->showbuttons) {
+		    gtk_widget_show(instance->image_fs);
+		    gtk_widget_show(instance->fs_event_box);
+		}
+	    }
 	    gtk_widget_show(instance->fixed_container);
 	    gtk_widget_show(instance->gtkwidget);
 
@@ -1125,7 +1223,8 @@ gboolean gtkgui_stop(void *data)
     instance->SetFullscreen(0);
     if (DEBUG > 1)
 	printf("completed gtkgui_stop\n");
-
+    gtkgui_resize(data);
+    gtk_widget_hide(instance->drawing_area);
     return FALSE;
 }
 
@@ -1158,12 +1257,12 @@ gboolean gtkgui_save_enable(void *data)
 #ifdef GTK2_ENABLED
 	filename = getURLFilename(instance->currentnode->url);
 	if (filename != NULL) {
-	    snprintf(newlabel, 1024, "Save as %s/%s",
+	    snprintf(newlabel, 1024, _("Save as %s/%s"),
 		     instance->download_dir, filename);
 	    NPN_MemFree(filename);
 
 	} else {
-	    snprintf(newlabel, 1024, "Save");
+	    snprintf(newlabel, 1024, _("Save"));
 	}
 
 	if (GTK_IS_BIN(instance->menuitem_save)) {
@@ -1178,7 +1277,7 @@ gboolean gtkgui_save_enable(void *data)
 				 instance->currentnode->retrieved);
     } else {
 #ifdef GTK2_ENABLED
-	snprintf(newlabel, 1024, "Append URL to %s/playlist",
+	snprintf(newlabel, 1024, _("Append URL to %s/playlist"),
 		 instance->download_dir);
 	label =
 	    GTK_LABEL(gtk_bin_get_child(GTK_BIN(instance->menuitem_save)));
@@ -1207,7 +1306,7 @@ gboolean gtkgui_drawMediaProgress(void *data)
 	if (GTK_IS_PROGRESS_BAR(instance->mediaprogress_bar)) {
 	    if ((instance->mediaPercent > 0)
 		&& (instance->mediaPercent <= 100)) {
-		if (instance->controlsvisible)
+		if (instance->controlsvisible && instance->showtracker)
 		    gtk_widget_show(GTK_WIDGET
 				    (instance->mediaprogress_bar));
 		gtk_progress_bar_update(instance->mediaprogress_bar,
@@ -1218,6 +1317,27 @@ gboolean gtkgui_drawMediaProgress(void *data)
 	}
     }
 
+    if (instance->movie_width != 0 && instance->movie_height != 0) {
+	if (GTK_IS_WIDGET(instance->progress_bar))
+	    gtk_widget_hide(GTK_WIDGET(instance->progress_bar));
+    }
+
+    return FALSE;
+}
+
+gboolean gtkgui_refreshbuttonstate(void *data) 
+{
+    nsPluginInstance *instance;
+
+    if (DEBUG > 1)
+	printf("in refreshbuttonstate method\n");
+
+    instance = (nsPluginInstance *) data;
+    if (instance == NULL)
+	return FALSE;
+    if (instance->mInitialized == FALSE)
+	return FALSE;
+
     return FALSE;
 }
 
@@ -1228,7 +1348,7 @@ gboolean gtkgui_updatebuttons(void *data)
     int win_height, win_width;
 
     if (DEBUG > 1)
-	printf("in drawMediaProgress method\n");
+	printf("in updatebuttons method\n");
 
     instance = (nsPluginInstance *) data;
 
@@ -1253,12 +1373,13 @@ gboolean gtkgui_updatebuttons(void *data)
 
 	if (instance->mmsstream == 0) {
 	    if (instance->rew_event_box != NULL) {
-		if (instance->showcontrols)
+		if (instance->showcontrols && instance->showbuttons) {
 		    gtk_widget_show(GTK_WIDGET(instance->rew_event_box));
-		gtk_fixed_move(GTK_FIXED(instance->fixed_container),
-			       GTK_WIDGET(instance->rew_event_box),
-			       width * multiplier++,
-			       instance->embed_height - height);
+		    gtk_fixed_move(GTK_FIXED(instance->fixed_container),
+				   GTK_WIDGET(instance->rew_event_box),
+				   width * multiplier++,
+				   instance->embed_height - height);
+		}
 	    }
 	} else {
 	    gtk_widget_hide(GTK_WIDGET(instance->rew_event_box));
@@ -1274,11 +1395,13 @@ gboolean gtkgui_updatebuttons(void *data)
 		       width * multiplier++, win_height - height);
 	if (instance->mmsstream == 0) {
 	    if (instance->ff_event_box != NULL) {
-		if (instance->showcontrols)
+		if (instance->showcontrols && instance->showbuttons) {
 		    gtk_widget_show(GTK_WIDGET(instance->rew_event_box));
-		gtk_fixed_move(GTK_FIXED(instance->fixed_container),
-			       GTK_WIDGET(instance->ff_event_box),
-			       width * multiplier++, win_height - height);
+		    gtk_fixed_move(GTK_FIXED(instance->fixed_container),
+				   GTK_WIDGET(instance->ff_event_box),
+				   width * multiplier++,
+				   win_height - height);
+		}
 	    }
 	} else {
 	    gtk_widget_hide(GTK_WIDGET(instance->ff_event_box));
@@ -1298,6 +1421,25 @@ gboolean gtkgui_updatebuttons(void *data)
     return FALSE;
 }
 
+gboolean gtkgui_updatefullscreen(void *data)
+{
+    nsPluginInstance *instance;
+
+    if (DEBUG > 1)
+	printf("in updatefullscreen method\n");
+
+    instance = (nsPluginInstance *) data;
+
+    if (instance == NULL)
+	return FALSE;
+    if (instance->mInitialized == FALSE)
+	return FALSE;
+
+    instance->SetFullscreen(instance->fullscreen);
+
+    return FALSE;
+}
+
 
 gint popup_handler(GtkWidget * widget, GdkEvent * event)
 {
@@ -1437,10 +1579,14 @@ void menuitem_copy_callback(GtkMenuItem * menuitem,
 			    nsPluginInstance * instance)
 {
     Node *n, *lastplayed;
+    char *localurl;
 #ifdef GTK2_ENABLED
     GtkClipboard *clipboard;
 #endif
-    char localurl[1024];
+#ifdef GTK1_ENABLED
+    GtkWidget *ge;
+#endif
+    localurl = (char *) malloc(1024);
 
     lastplayed = NULL;
 
@@ -1462,10 +1608,390 @@ void menuitem_copy_callback(GtkMenuItem * menuitem,
 	gtk_clipboard_set_text(clipboard, localurl, strlen(localurl));
 #endif
 #ifdef GTK1_ENABLED
-	XStoreBytes(instance->display, localurl, strlen(localurl));
+	ge = gtk_entry_new_with_max_length(strlen(localurl));
+	gtk_entry_set_text((GtkEntry *) ge, localurl);
+	gtk_editable_select_region((GtkEditable *) ge, 0,
+				   strlen(localurl));
+	gtk_editable_copy_clipboard((GtkEditable *) ge);
 #endif
     }
 }
 
+gint CloseConfig(GtkWidget * widget, nsPluginInstance * instance)
+{
 
-#endif
+    if (GTK_IS_WIDGET(instance->conf_window))
+	gtk_widget_destroy(instance->conf_window);
+    instance->conf_window = NULL;
+
+    return (FALSE);
+
+}
+
+gint CloseConfigEvent(GtkWidget * widget, GdkEvent *event, nsPluginInstance * instance)
+{
+
+    if (GTK_IS_WIDGET(instance->conf_window)) 
+	gtk_widget_destroy(instance->conf_window);
+    instance->conf_window = NULL;
+    return (FALSE);
+
+}
+
+#ifdef GTK2_ENABLED
+gint ApplyConfig(GtkWidget * widget, nsPluginInstance * instance)
+{
+    FILE *input;
+    FILE *output;
+    char config_name[1000];
+    char tmp_name[1000];
+    char buffer[1000];
+    int ret;
+
+    instance->cachesize =
+	(int) gtk_range_get_value(GTK_RANGE(instance->conf_cachesize));
+    instance->cache_percent =
+	(int) gtk_range_get_value(GTK_RANGE(instance->conf_cachepercent));
+
+    if (instance->vo != NULL)
+	free(instance->vo);
+    instance->vo =
+	strdup(gtk_entry_get_text
+	       (GTK_ENTRY(GTK_BIN(instance->conf_vo)->child)));
+
+    if (instance->ao != NULL)
+	free(instance->ao);
+    instance->ao =
+	strdup(gtk_entry_get_text
+	       (GTK_ENTRY(GTK_BIN(instance->conf_ao)->child)));
+
+    if (instance->download_dir != NULL)
+	free(instance->download_dir);
+    instance->download_dir =
+	strdup(gtk_entry_get_text
+	       (GTK_ENTRY(GTK_BIN(instance->conf_dir)->child)));
+
+    gtk_widget_destroy(instance->conf_window);
+    instance->conf_window = NULL;
+
+    // write the setting to the conf file now
+
+    snprintf(config_name, 1000, "%s", getenv("HOME"));
+    strlcat(config_name, "/.mplayer/mplayerplug-in.conf", 1000);
+
+    snprintf(tmp_name, 1000, "%s", getenv("HOME"));
+    strlcat(tmp_name, "/.mplayer/mplayerplug-in.conf.tmp", 1000);
+
+    if (DEBUG) {
+	printf("config %s\n", config_name);
+	printf("tmp %s\n", tmp_name);
+
+    }
+
+    input = fopen(config_name, "r");
+    output = fopen(tmp_name, "w");
+
+    if (input != NULL && output != NULL) {
+	if (DEBUG)
+	    printf("update existing config\n");
+	if (strlen(instance->vo) > 0)
+	    fprintf(output, "vo=%s\n", instance->vo);
+	if (strlen(instance->ao) > 0)
+	    fprintf(output, "ao=%s\n", instance->ao);
+	fprintf(output, "cachesize=%i\n", instance->cachesize);
+	fprintf(output, "cache-percent=%i\n", instance->cache_percent);
+	if (strlen(instance->download_dir) > 0)
+	    fprintf(output, "dload-dir=%s\n", instance->download_dir);
+
+	while (fgets(buffer, sizeof(buffer), input) != NULL) {
+	    if (strncasecmp(buffer, "vo", 2) == 0) {
+	    } else if (strncasecmp(buffer, "ao", 2) == 0) {
+	    } else if (strncasecmp(buffer, "cachesize", 9) == 0) {
+	    } else if (strncasecmp(buffer, "cache-percent", 13) == 0) {
+	    } else if (strncasecmp(buffer, "dload-dir", 9) == 0) {
+	    } else {
+		fprintf(output, "%s", buffer);
+	    }
+
+	}
+	fclose(input);
+	fclose(output);
+	ret = unlink(config_name);
+	if (ret == 0) {
+	    ret = rename(tmp_name, config_name);
+	    if (ret == -1) {
+		ret = errno;
+		if (DEBUG) {
+		    printf("Unable to rename the file %s to %s\n",
+			   tmp_name, config_name);
+		    printf("Error code: %i - %s\n", ret, strerror(ret));
+		}
+	    }
+	} else {
+	    ret = errno;
+	    if (DEBUG) {
+		printf("Unable to delete the file %s\n", config_name);
+		printf("Error code: %i - %s\n", ret, strerror(ret));
+	    }
+	}
+    } else {
+	if (DEBUG)
+	    printf("write new config file\n");
+	if (output != NULL) {
+	    if (strlen(instance->vo) > 0)
+		fprintf(output, "vo=%s\n", instance->vo);
+	    if (strlen(instance->ao) > 0)
+		fprintf(output, "ao=%s\n", instance->ao);
+	    fprintf(output, "cachesize=%i\n", instance->cachesize);
+	    fprintf(output, "cache-percent=%i\n", instance->cache_percent);
+	    if (strlen(instance->download_dir) > 0)
+		fprintf(output, "dload-dir=%s\n", instance->download_dir);
+	    fclose(output);
+	    ret = rename(tmp_name, config_name);
+	    if (ret == -1) {
+		ret = errno;
+		if (DEBUG) {
+		    printf("Unable to rename the file %s to %s\n",
+			   tmp_name, config_name);
+		    printf("Error code: %i - %s\n", ret, strerror(ret));
+		}
+	    }
+	}
+
+    }
+    return (FALSE);
+}
+
+void menuitem_config_callback(GtkMenuItem * menuitem,
+			      nsPluginInstance * instance)
+{
+    GtkWidget *conf_vbox;
+    GtkWidget *conf_hbutton_box;
+    GtkWidget *conf_ok;
+    GtkWidget *conf_cancel;
+    GtkWidget *conf_table;
+    GtkWidget *conf_label;
+
+    if (instance->conf_window != NULL)
+	return;
+
+    instance->conf_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    gtk_widget_realize(instance->conf_window);
+    gtk_window_set_resizable(GTK_WINDOW(instance->conf_window), FALSE);
+
+    conf_vbox = gtk_vbox_new(FALSE, 10);
+    conf_hbutton_box = gtk_hbutton_box_new();
+    conf_table = gtk_table_new(10, 2, FALSE);
+
+    gtk_container_add(GTK_CONTAINER(conf_vbox), conf_table);
+    gtk_container_add(GTK_CONTAINER(conf_vbox), conf_hbutton_box);
+    gtk_container_add(GTK_CONTAINER(instance->conf_window), conf_vbox);
+    gtk_window_set_title(GTK_WINDOW(instance->conf_window),
+			 "mplayerplug-in configuration");
+    gtk_container_set_border_width(GTK_CONTAINER(instance->conf_window),
+				   5);
+    g_signal_connect(GTK_OBJECT(instance->conf_window), "delete_event",
+		     GTK_SIGNAL_FUNC(CloseConfigEvent), instance);
+
+    instance->conf_vo = gtk_combo_box_entry_new_text();
+    if (instance->conf_vo != NULL) {
+	gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_vo), "x11");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_vo), "xv");
+	if (instance->vo != NULL) {
+	    if (strcmp(instance->vo, "x11") == 0)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_vo),
+					 0);
+	    if (strcmp(instance->vo, "xv") == 0)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_vo),
+					 1);
+	    if (gtk_combo_box_get_active(GTK_COMBO_BOX(instance->conf_vo))
+		== -1) {
+		gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_vo),
+					  instance->vo);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_vo),
+					 2);
+	    }
+	}
+    }
+
+    instance->conf_ao = gtk_combo_box_entry_new_text();
+    if (instance->conf_ao != NULL) {
+	gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_ao),
+				  "alsa");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_ao),
+				  "arts");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_ao), "esd");
+	gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_ao), "oss");
+	if (instance->ao != NULL) {
+	    if (strcmp(instance->ao, "alsa") == 0)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_ao),
+					 0);
+	    if (strcmp(instance->ao, "arts") == 0)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_ao),
+					 1);
+	    if (strcmp(instance->ao, "esd") == 0)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_ao),
+					 2);
+	    if (strcmp(instance->ao, "oss") == 0)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_ao),
+					 3);
+	    if (gtk_combo_box_get_active(GTK_COMBO_BOX(instance->conf_ao))
+		== -1) {
+		gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_ao),
+					  instance->ao);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_ao),
+					 4);
+	    }
+	}
+    }
+
+    instance->conf_dir = gtk_combo_box_entry_new_text();
+    if (instance->conf_dir != NULL) {
+	gtk_combo_box_append_text(GTK_COMBO_BOX(instance->conf_dir),
+				  "$HOME");
+	if (instance->download_dir != NULL) {
+	    if (strcmp(instance->download_dir, "$HOME") == 0)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_dir),
+					 0);
+	    if (gtk_combo_box_get_active(GTK_COMBO_BOX(instance->conf_dir))
+		== -1) {
+		gtk_combo_box_append_text(GTK_COMBO_BOX
+					  (instance->conf_dir),
+					  instance->download_dir);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(instance->conf_dir),
+					 1);
+	    }
+	}
+    }
+
+    conf_ok = gtk_button_new_with_label(_("OK"));
+    g_signal_connect(GTK_OBJECT(conf_ok), "clicked",
+		     GTK_SIGNAL_FUNC(ApplyConfig), instance);
+
+    conf_cancel = gtk_button_new_with_label(_("Cancel"));
+    g_signal_connect(GTK_OBJECT(conf_cancel), "clicked",
+			     GTK_SIGNAL_FUNC(CloseConfig),
+			     instance);
+
+    conf_label = gtk_label_new(_("Video Output:"));
+    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_table_attach_defaults(GTK_TABLE(conf_table), conf_label, 0, 1, 0,
+			      1);
+    gtk_widget_show(conf_label);
+    gtk_table_attach_defaults(GTK_TABLE(conf_table), instance->conf_vo, 1,
+			      2, 0, 1);
+    conf_label = gtk_label_new(_("Audio Output:"));
+    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_table_attach_defaults(GTK_TABLE(conf_table), conf_label, 0, 1, 1,
+			      2);
+    gtk_widget_show(conf_label);
+    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_table_attach_defaults(GTK_TABLE(conf_table), instance->conf_ao, 1,
+			      2, 1, 2);
+    conf_label = gtk_label_new(_("Save to Location:"));
+    gtk_table_attach_defaults(GTK_TABLE(conf_table), conf_label, 0, 1, 2,
+			      3);
+    gtk_widget_show(conf_label);
+    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_table_attach_defaults(GTK_TABLE(conf_table), instance->conf_dir, 1,
+			      2, 2, 3);
+
+
+    conf_label = gtk_label_new(_("Minimum Cache Size:"));
+    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_table_attach_defaults(GTK_TABLE(conf_table), conf_label, 0, 1, 3,
+			      4);
+    gtk_widget_show(conf_label);
+    instance->conf_cachesize = gtk_hscale_new_with_range(0, 32767, 512);
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_cachesize, 1, 2, 3, 4);
+    gtk_range_set_value(GTK_RANGE(instance->conf_cachesize),
+			instance->cachesize);
+    gtk_widget_show(instance->conf_cachesize);
+
+    conf_label = gtk_label_new(_("Percent of Media to Cache:"));
+    gtk_misc_set_alignment (GTK_MISC(conf_label), 0.0, 0.0);
+    gtk_table_attach_defaults(GTK_TABLE(conf_table), conf_label, 0, 1, 4,
+			      5);
+    gtk_widget_show(conf_label);
+    instance->conf_cachepercent = gtk_hscale_new_with_range(0, 100, 5);
+    gtk_table_attach_defaults(GTK_TABLE(conf_table),
+			      instance->conf_cachepercent, 1, 2, 4, 5);
+    gtk_range_set_value(GTK_RANGE(instance->conf_cachepercent),
+			instance->cache_percent);
+    gtk_widget_show(instance->conf_cachepercent);
+
+    gtk_container_add(GTK_CONTAINER(conf_hbutton_box), conf_ok);
+    gtk_container_add(GTK_CONTAINER(conf_hbutton_box), conf_cancel);
+
+
+    if (instance->conf_vo != NULL)
+	gtk_widget_show(instance->conf_vo);
+    if (instance->conf_ao != NULL)
+	gtk_widget_show(instance->conf_ao);
+    if (instance->conf_dir != NULL)
+	gtk_widget_show(instance->conf_dir);
+
+    gtk_widget_show(conf_ok);
+    gtk_widget_show(conf_cancel);
+    gtk_widget_show(conf_table);
+    gtk_widget_show(conf_hbutton_box);
+    gtk_widget_show(conf_vbox);
+    gtk_widget_show(instance->conf_window);
+
+}
+
+gboolean window_visible(GtkWidget * widget, GdkEvent * event,
+			nsPluginInstance * instance)
+{
+
+    //if (DEBUG)
+    //      printf("**************\n\n\n\n in window_visible %i\n\n\n\n\n******************\n",GTK_WIDGET_VISIBLE(widget));
+
+    // only pause if video is present, if we have streaming audio we want to keep playing
+
+    if (event->type == GDK_VISIBILITY_NOTIFY) {
+
+	if (((GdkEventVisibility *) event)->state ==
+	    GDK_VISIBILITY_FULLY_OBSCURED) {
+	    if (instance->paused == 0 && instance->threadsignaled
+		&& instance->nopauseonhide == 0) {
+
+		if (instance->movie_width != 0
+		    && instance->movie_height != 0) {
+		    instance->Pause();
+		    instance->paused_wheninvisible = 1;
+		}
+	    }
+
+	    if (instance->onHiddenCallback != NULL)
+		NPN_GetURL(instance->mInstance,
+			   instance->onHiddenCallback, "_self");
+
+	} else {
+	    if (instance->paused_wheninvisible == 1
+		&& instance->threadsignaled
+		&& instance->nopauseonhide == 0) {
+
+		if (instance->movie_width != 0
+		    && instance->movie_height != 0) {
+		    instance->Play();
+		    instance->paused_wheninvisible = 0;
+		}
+	    }
+
+	    if (instance->onVisibleCallback != NULL)
+		NPN_GetURL(instance->mInstance,
+			   instance->onVisibleCallback, "_self");
+
+	}
+
+    }
+
+    return (FALSE);
+
+}
+
+#endif				// GTK2_ENABLED
+
+#endif				// GTK_ENABLED
