@@ -37,11 +37,16 @@
 
 #ifndef __PLUGIN_H__
 #define __PLUGIN_H__
+#ifdef BSD
+#include <cmath>
+#endif
 #define _XOPEN_SOURCE 500
 
+#ifndef BSD
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
+#endif
 #ifdef GECKOSDK_ENABLED
 #include "mozilla-config.h"
 #endif
@@ -51,6 +56,13 @@
 #include <pthread.h>
 #include <sys/types.h>
 #include <signal.h>
+#include <string.h>
+#ifdef HAVE_GETTEXT
+# include <libintl.h>
+# define _(x) (gettext(x))
+#else
+# define _(x) (x)
+#endif
 
 class nsPluginInstance:public nsPluginInstanceBase {
   public:
@@ -59,16 +71,19 @@ class nsPluginInstance:public nsPluginInstanceBase {
 
     NPBool init(NPWindow * aWindow);
     void shut();
+    void shutdown();
     NPBool isInitialized();
     NPError NewStream(NPMIMEType type, NPStream * stream, NPBool seekable,
 		      uint16 * stype);
     NPError SetWindow(NPWindow * aWindow);
     NPError DestroyStream(NPStream * stream, NPError reason);
+    void URLNotify(const char *url, NPReason reason, void *notifyData);
     int32 WriteReady(NPStream * stream);
     int32 Write(NPStream * stream, int32 offset, int32 len, void *buffer);
 
     // JS Methods
     void Play();
+    void PlayAt(double counter);
     void Pause();
     void Stop();
     void Quit();
@@ -87,6 +102,12 @@ class nsPluginInstance:public nsPluginInstanceBase {
     void SetFullscreen(PRBool value);
     void GetShowlogo(PRBool *_retval);
     void SetShowlogo(PRBool value);
+    void GetAutoPlay(PRBool *_retval);
+    void SetAutoPlay(PRBool value);
+    void GetLoop(PRBool *_retval);
+    void SetLoop(PRBool value);
+    void GetMIMEType(char **_retval);
+    void GetPlaying(PRBool *_retval);
     
     // we need to provide implementation of this method as it will be
     // used by Mozilla to retrive the scriptable peer
@@ -124,13 +145,16 @@ class nsPluginInstance:public nsPluginInstanceBase {
     pid_t pid;
     int noredraw;
     int hrefrequested;
-    int threadsetup;
-    int threadlaunched;
-    int threadsignaled;
+    int threadsetup;			// has the child thread been setup but not signalled
+    int threadlaunched;			// only allow 1 child thread per instance
+    int threadsignaled;			// keep track if the thread has been told to start
     int cancelled;
     int autostart;
     int controlwindow;
     int showcontrols;
+    int showtracker;
+    int showbuttons;
+    int showfsbutton;
     int mmsstream;
     Node *list;
     Node *currentnode;
@@ -145,6 +169,9 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int panel_drawn;
     float percent;
     char *mediaCompleteCallback;
+    char *onVisibleCallback;
+    char *onHiddenCallback;
+    char *onDestroyCallback;
     float mediaLength;			// length of media in seconds
     int mediaPercent;			// percentage of media played
     float mediaTime; 			// time in seconds
@@ -154,11 +181,18 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int showlogo;
     int DPMSEnabled;
     int hidden;
+    int black_background;
+    int nomouseinput;
+    int noconsolecontrols;
+    int cookies;
+    long int starttime;
+    int nopauseonhide;
     
 #ifdef GTK_ENABLED
     GtkWidget *gtkwidget;
     GtkWidget *image;
     GtkWidget *fixed_container;
+    GtkWidget *fixed_event_box;
     GtkProgressBar *progress_bar;
     GtkProgressBar *mediaprogress_bar;
     GtkLabel *status;
@@ -171,7 +205,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GtkWidget *ff_event_box;
     GtkWidget *rew_event_box;
     GtkWidget *fs_event_box;
-
+    
     GtkWidget *image_play;
     GtkWidget *image_pause;
     GtkWidget *image_stop;
@@ -185,10 +219,19 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GtkMenuItem *menuitem_stop;
     GtkMenuItem *menuitem_sep1;
     GtkMenuItem *menuitem_sep2;
+    GtkMenuItem *menuitem_sep3;
     GtkMenuItem *menuitem_showcontrols;
     GtkMenuItem *menuitem_fullscreen;
     GtkMenuItem *menuitem_save;
     GtkMenuItem *menuitem_copy;
+    GtkMenuItem *menuitem_config;
+
+    GtkWidget *conf_window;
+	GtkWidget *conf_vo;
+	GtkWidget *conf_ao;
+	GtkWidget *conf_dir;
+	GtkWidget *conf_cachesize;
+	GtkWidget *conf_cachepercent;
     
 #endif
 
@@ -215,7 +258,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GdkPixbuf *pb_sm_rew_down;
     GdkPixbuf *pb_sm_fs_up;
     GdkPixbuf *pb_sm_fs_down;
-
+    int paused_wheninvisible;
 #endif
 
 #ifdef GTK1_ENABLED
@@ -276,6 +319,10 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int toolkitok;
     int moz_toolkit;
     int plug_toolkit;
+    int framedrop;
+    int autosync;
+    int mc;
+    char *useragent;
     
     // JavaScript State
     int paused;
